package org.apache.maven.plugin.testing;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.codehaus.plexus.util.StringUtils;

/**
 * @author Edwin Punzalan
 * @version $Id: ExpressionEvaluatorMojo.java 638332 2008-03-18 11:39:00Z bentmann $
 */
public class ExpressionEvaluatorMojo
    extends AbstractMojo
{
    private String basedir;

    private ArtifactRepository localRepository;

    private String workdir;

    /** {@inheritDoc} */
    public void execute()
        throws MojoExecutionException, MojoFailureException
    {
        if ( StringUtils.isEmpty( basedir ) )
        {
            throw new MojoExecutionException( "basedir was not injected." );
        }

        if ( localRepository == null )
        {
            throw new MojoExecutionException( "localRepository was not injected." );
        }

        if ( StringUtils.isEmpty( workdir ) )
        {
            throw new MojoExecutionException( "workdir was not injected." );
        }
        else if ( !workdir.startsWith( basedir ) )
        {
            throw new MojoExecutionException( "workdir does not start with basedir." );
        }
    }
}
